@@ -25,9 +25,10 @@ from WebStack.Generic import ContentType, EndOfResponse
 import XSLForms.Resources.WebResources
 import XSLForms.Utils
 from XSLTools import XMLCalendar
+from WebCalendar.Common import rdfcalendar_item_types, rdfcalendar_calendar_item_types
 import os
 import time
-import calendar
+import urllib
 
 # Resource classes.
 
@@ -40,12 +41,14 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
     template_resources = {
         "event" : ("event_template.xhtml", "event_output.xsl"),
         "to-do" : ("to_do_template.xhtml", "to_do_output.xsl"),
-        "card" : ("card_template.xhtml", "card_output.xsl")
+        "card" : ("card_template.xhtml", "card_output.xsl"),
+        "journal" : ("journal_template.xhtml", "journal_output.xsl")
         }
     init_resources = {
         "event" : ("event_template.xhtml", "event_input.xsl"),
         "to-do" : ("to_do_template.xhtml", "to_do_input.xsl"),
-        "card" : ("card_template.xhtml", "card_input.xsl")
+        "card" : ("card_template.xhtml", "card_input.xsl"),
+        "journal" : ("journal_template.xhtml", "journal_input.xsl")
         }
     transform_resources = {
         "event" : ["event_prepare.xsl"],
@@ -55,7 +58,8 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
         "event-calendar" : ["event_calendar_mark.xsl"]
         }
     in_page_resources = {
-        "datetimes" : ("event", "event_datetimes_output.xsl", "datetimes-node")
+        "event-datetimes" : ("event", "event_datetimes_output.xsl", "datetimes-node"),
+        "journal-datetimes" : ("journal", "journal_datetimes_output.xsl", "datetimes-node")
     }
     document_resources = {
         "adr" : "card_adr.xml",
@@ -102,17 +106,16 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
             trans.get_response_stream().write("Item type not understood")
             raise EndOfResponse
 
-        # Useful information.
-
-        time_now = time.strftime("%Y%m%dT%H%M%S")
-        last_modified_dates = doc.xpath("item/*/last-modified")
-        if last_modified_dates:
-            last_modified_dates[0].setAttribute("datetime", time_now)
-
         # Perform any actions.
 
         parameters = form.get_parameters()
         if parameters.has_key("save"):
+
+            # Add non-user-supplied information.
+
+            self._add_related_to(doc, attributes)
+            self._add_dates(doc, item)
+
             if self.save_document(doc, item, item_type):
                 trans.redirect(trans.encode_path(parent_path, self.path_encoding))
                 # Response ended!
@@ -181,7 +184,7 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
         dtstart_month, start_date = self._get_month(doc, "/item/*/dtstart/month")
         dtend_month, end_date = self._get_month(doc, "/item/*/dtend/month")
 
-        # Where no start or end dates exists, use the current month.
+        # Get the current date and time.
 
         year_now, month_now, day_now = time.localtime()[:3]
         time_in_day_now = "" # time.strftime("%H%M%S")
@@ -189,18 +192,13 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
         # Since the dtstart and dtend datetimes are removed after navigating
         # away from them in the month navigator, the month start/end values
         # can be derived from dtstart/dtend if they are present, or taken
-        # directly from the document if they are not. Where no values are
-        # present, use the current date.
+        # directly from the document if they are not.
 
         if dtstart_date is not None:
             start_date = (dtstart_date[0], dtstart_date[1])
-        elif start_date is None:
-            start_date = (year_now, month_now)
 
         if dtend_date is not None:
             end_date = (dtend_date[0], dtend_date[1])
-        elif end_date is None:
-            end_date = (year_now, month_now)
 
         # Detect the month navigation actions.
         # Remove any datetime set through month navigation - this makes the user set the
@@ -208,18 +206,26 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
         # in-page updates.
 
         if parameters.has_key("previous-start"):
-            start_date = self._previous_month(start_date)
+            start_date, _tmp = self._previous_month(start_date)
             dtstart_date, dtstart_time = None, None
         elif parameters.has_key("next-start"):
-            start_date = self._next_month(start_date, end_date)
+            start_date, end_date = self._next_month(start_date, end_date)
             dtstart_date, dtstart_time = None, None
         elif parameters.has_key("previous-end"):
-            end_date = self._previous_month(end_date, start_date)
+            end_date, start_date = self._previous_month(end_date, start_date)
             dtend_date, dtend_time = None, None
         elif parameters.has_key("next-end"):
-            end_date = self._next_month(end_date)
+            end_date, _tmp = self._next_month(end_date)
             dtend_date, dtend_time = None, None
 
+        # Where no month navigation values are present, use the current date.
+
+        if start_date is None:
+            start_date = (year_now, month_now)
+
+        if end_date is None:
+            end_date = (year_now, month_now)
+
         # Update the datetimes. Here, we remove any existing month data, update
         # the datetime attribute or remove it if month navigation has occurred.
 
@@ -232,6 +238,10 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
                 dtstart.removeAttribute("datetime")
             XMLCalendar.write_month_to_document(doc, dtstart, start_date[0], start_date[1])
 
+            # Add the current date for informational purposes.
+
+            dtstart.setAttribute("today", "%04d%02d%02d" % (year_now, month_now, day_now))
+
         if dtend is not None:
             if dtend_month is not None:
                 dtend_month.parentNode.removeChild(dtend_month)
@@ -241,6 +251,10 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
                 dtend.removeAttribute("datetime")
             XMLCalendar.write_month_to_document(doc, dtend, end_date[0], end_date[1])
 
+            # Add the current date for informational purposes.
+
+            dtend.setAttribute("today", "%04d%02d%02d" % (year_now, month_now, day_now))
+
         # Process the calendar, adding marked, start and end attributes.
 
         if dtstart is not None:
@@ -295,7 +309,7 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
 
         # Or obtain the item definition.
 
-        elif item_type in ("event", "to-do", "card"):
+        elif item_type in rdfcalendar_item_types:
             doc = form.new_instance("item")
             doc_root = doc.xpath("*")[0]
 
@@ -308,7 +322,7 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
                 # This moves certain elements inside others and merges person
                 # information into the right elements.
 
-                if item_type in ("event", "to-do"):
+                if item_type in rdfcalendar_calendar_item_types:
 
                     # Get the person definitions.
 
@@ -325,13 +339,19 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
 
     def save_document(self, doc, item, item_type):
 
+        """
+        Save the given document 'doc' using the 'item' and 'item_type'
+        information to first process the document before presenting it to the
+        data store.
+        """
+
         # Restructure the serialised form, merging fields which have to be
         # combined into single values.
 
         if item_type == "card":
             unprepare_xsl_list = self.prepare_transform("card-unprepare")
             doc = self.get_result(unprepare_xsl_list, doc)
-        elif item_type in ("event", "to-do"):
+        elif item_type in ("event", "to-do", "journal"):
             unprepare_xsl_list = self.prepare_transform("event-unprepare")
             doc = self.get_result(unprepare_xsl_list, doc)
 
@@ -341,6 +361,7 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
             self.store.remove_item(item)
 
         # Where no existing item exists, try and find a reasonable identifier.
+        # Do not allow items to be saved without a proper item reference.
 
         elif item_type == "card":
             uriref_values = doc.xpath("/item/*//email/@details")
@@ -348,9 +369,22 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
                 uriref_value = uriref_values[0].nodeValue
                 if uriref_value:
                     item = self.store.URIRef("MAILTO:" + uriref_value)
+                else:
+                    return 0
+            else:
+                return 0
 
-        # Do not allow items to be saved without a proper item reference.
-
+        elif item_type in ("event", "to-do", "journal"):
+            summary_values = doc.xpath("/item/*/summary/@details")
+            if len(summary_values) > 0:
+                summary_value = summary_values[0].nodeValue
+                if summary_value:
+                    # NOTE: This will not work with various leading characters.
+                    # Change the summary into a URL-encoded identifier.
+                    item = self.store.get_item_from_identifier(urllib.quote(summary_value))
+                    while self.store.has_item(item):
+                        summary_value += "'"
+                        item = self.store.get_item_from_identifier(urllib.quote(summary_value))
                 else:
                     return 0
             else:
@@ -361,6 +395,41 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
         self.store.add_item(item, doc)
         return 1
 
+    # Methods adding information for saving.
+
+    def _add_dates(self, doc, item):
+
+        """
+        Add date information to the given document 'doc', providing creation
+        date information if the specified 'item' is None.
+        """
+
+        time_now = time.strftime("%Y%m%dT%H%M%SZ", time.gmtime())
+        last_modified_dates = doc.xpath("item/*/last-modified")
+        if last_modified_dates:
+            last_modified_dates[0].setAttribute("datetime", time_now)
+        if item is None:
+            created_dates = doc.xpath("item/*/created")
+            if created_dates:
+                created_dates[0].setAttribute("datetime", time_now)
+            dtstamp_dates = doc.xpath("item/*/dtstamp")
+            if dtstamp_dates:
+                dtstamp_dates[0].setAttribute("datetime", time_now)
+
+    def _add_related_to(self, doc, attributes):
+
+        """
+        Add related-to information to the given document 'doc', using the
+        'attributes' provided to supply the necessary details.
+        """
+
+        # NOTE: To be improved.
+        related_tos = doc.xpath("/item/*/related-to")
+        if related_tos and (not related_tos[0].hasAttribute("uri") or not related_tos[0].getAttribute("uri")) and \
+            attributes.get("filter-type") == "related-to":
+            related_to = attributes["filter-item-value"]
+            related_tos[0].setAttribute("uri", unicode(self.store.get_item_from_identifier(related_to)))
+
     # Datetime-related methods.
 
     def _get_dtelement(self, doc, name_path):
@@ -391,23 +460,25 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
         year, month = date
         if limit_date is not None:
             limit_year, limit_month = limit_date
-        if limit_date is None or (year, month) > (limit_year, limit_month):
-            month -= 1
-            if month == 0:
-                month = 12
-                year -= 1
-        return year, month
+        month -= 1
+        if month == 0:
+            month = 12
+            year -= 1
+        if limit_date is not None and (year, month) < (limit_year, limit_month):
+            limit_date = year, month
+        return (year, month), limit_date
 
     def _next_month(self, date, limit_date=None):
         year, month = date
         if limit_date is not None:
             limit_year, limit_month = limit_date
-        if limit_date is None or (year, month) < (limit_year, limit_month):
-            month += 1
-            if month == 13:
-                month = 1
-                year += 1
-        return year, month
+        month += 1
+        if month == 13:
+            month = 1
+            year += 1
+        if limit_date is not None and (year, month) > (limit_year, limit_month):
+            limit_date = year, month
+        return (year, month), limit_date
 
     # Person initialisation methods.
 
@@ -457,9 +528,10 @@ class ItemEditorResource(XSLForms.Resources.WebResources.XSLFormsResource):
 
         if value != "":
             suggestion_items = self.store.get_filtered_items([
-                (["uri"], [("startswith", value)]),
-                (["email", "details"], [("startswith", value)]),
-                (["fn", "details"], [("startswith", value)])
+                (["organizer", "uri"], ("startswith", value)),
+                (["attendee", "uri"], ("startswith", value)),
+                (["email", "details"], ("startswith", value)),
+                (["fn", "details"], ("startswith", value))
                 ])
         else:
             suggestion_items = []
