@@ -97,25 +97,30 @@ class Store:
 
     # Low-level query methods.
 
-    def get_values_for_property(self, property_type, property_value=None, item=None, comparison=None):
+    def get_comparison(self, comparison, values):
 
         """
-        Return values having the given 'property_type' and matching the
-        specified 'property_value' for the given 'item'.
-
-        If 'item' is None, all nodes will be searched for properties of the
-        given 'property_type'. If 'property_value' is None, all values will be
-        returned for the given 'item' (or all items).
-
-        If the optional 'comparison' is specified using a string method name,
-        each item's value for the stated 'property_type' will be asked to
-        provide a method of that name, and if the given 'property_value' passed
-        as a parameter to that method causes a true value to be returned, the
-        value will be added to the result list.
+        Return a comparison expression for the given 'comparison' and 'value',
+        suitable for triple stores which support querying.
         """
 
-        for _item, _property_type, _property_value in self.get_triples_with_property(property_type, property_value, item, comparison=comparison):
-            yield _property_value
+        # NOTE: Permits precise matching of URIRefs.
+
+        if len(values) == 1 and comparison is None:
+            return values[0]
+
+        # NOTE: Permits literal matching.
+
+        expressions = []
+        object_type = "U"
+        for value in values:
+            if not isinstance(value, self.URIRef):
+                object_type = "L"
+            if comparison == "startswith":
+                expressions.append("substr(_, 1, %s) = ?" % len(value))
+            else:
+                expressions.append("_ = ?")
+        return self.cstore.store.Expression("(" + " or ".join(expressions) + ")", values, object_type)
 
     def get_items_with_property(self, property_type, property_value, comparison=None):
 
@@ -134,34 +139,20 @@ class Store:
 
     def get_triples_with_property(self, property_type, property_value, item, comparison=None):
         if comparison is None:
-            for triple in self.cstore.store.triples((item, property_type, property_value)):
-                yield triple
+            for _item, _property_type, _property_value in self.cstore.store.triples((item, property_type, property_value)):
+                yield _item, _property_type, _property_value
         else:
             for _item, _property_type, _property_value in self.cstore.store.triples((item, property_type, None)):
-                if hasattr(_property_value, comparison) and getattr(_property_value, comparison)(property_value):
+                if property_value is None or \
+                    comparison is None and _property_value == property_value or \
+                    comparison is not None and hasattr(_property_value, comparison) and getattr(_property_value, comparison)(property_value):
                     yield _item, _property_type, _property_value
 
-    def get_values_for_properties(self, property_types, property_value, comparison=None):
-
-        """
-        Return all values belonging to nodes having the given 'property_types'
-        and final 'property_value'.
-
-        If the optional 'comparison' is specified using a string method name,
-        each item's value for the stated 'property_type' will be asked to
-        provide a method of that name, and if the given 'property_value' passed
-        as a parameter to that method causes a true value to be returned, the
-        item will be used to find suitable owner items.
-        """
-
-        for _item, _property_value in self.get_owner_items_and_values_with_properties(property_types, property_value, comparison):
-            yield _property_value
-
-    def get_owner_items_with_properties(self, property_types, property_value, comparison=None):
+    def get_owner_items_with_properties(self, property_types, property_values, comparison=None):
 
         """
         Return all items containing nodes having the given 'property_types' and
-        final 'property_value'.
+        final 'property_values'.
 
         If the optional 'comparison' is specified using a string method name,
         each item's value for the stated 'property_type' will be asked to
@@ -170,24 +161,44 @@ class Store:
         item will be used to find suitable owner items.
         """
 
-        for _item, _property_value in self.get_owner_items_and_values_with_properties(property_types, property_value, comparison):
-            yield _item
-
-    def get_owner_items_and_values_with_properties(self, property_types, property_value, comparison=None):
         reversed_types = property_types[:]
         reversed_types.reverse()
 
-        for item, _property_type, _property_value in self.get_triples_with_property(reversed_types[0], property_value, None, comparison=comparison):
-            current_item = item
-            for property_type in reversed_types[1:]:
-                owner_item = None
-                for owner_item in self.get_items_with_property(property_type, current_item):
-                    break
-                current_item = owner_item
-                if current_item is None:
-                    break
-            if current_item is not None:
-                yield current_item, _property_value
+        for property_value in property_values:
+            for item, _property_type, _property_value in self.get_triples_with_property(reversed_types[0], property_value, None, comparison=comparison):
+                current_item = item
+                for property_type in reversed_types[1:]:
+                    owner_item = None
+                    for owner_item in self.get_items_with_property(property_type, current_item):
+                        break
+                    current_item = owner_item
+                    if current_item is None:
+                        break
+                if current_item is not None:
+                    yield current_item
+
+    def get_owner_items_using_query(self, property_types, property_values, comparison):
+        reversed_types = property_types[:]
+        reversed_types.reverse()
+
+        # NOTE: Permits negation.
+
+        if len(property_values) == 0:
+            query = self.get_comparison(None, [self.cstore.store.Defined()])
+        else:
+            query = self.get_comparison(comparison, property_values)
+
+        # Build the subselect hierarchy.
+
+        for property_type in reversed_types:
+            query = self.cstore.store.subjects(property_type, query)
+
+        # NOTE: The actual negation.
+
+        if len(property_values) == 0:
+            query = self.cstore.store.subjects(pattern=[self.cstore.store.negation(query)])
+
+        return query
 
     def get_item_type(self, item):
 
@@ -285,6 +296,12 @@ class Store:
         self.cstore.remove_node(item, deep)
         self.commit()
 
+    def has_item(self, item):
+
+        "Return whether the store contains the given 'item'."
+
+        return self.cstore.has_triple((item, None, None))
+
     # Conversions from items to identifiers and back again.
 
     def get_item_from_identifier(self, identifier):
@@ -367,7 +384,10 @@ class Store:
         if identifier == self.TYPE:
             return "item-type"
         else:
-            return self.cstore.get_label(identifier)
+            label = self.cstore.get_label(identifier)
+            if label is None:
+                label = identifier.split("/")[-1]
+            return label
 
     def get_uid(self, identifier):
 
@@ -418,6 +438,8 @@ class Store:
     def get_property_type_from_identifier(self, identifier):
         if identifier == "item-type":
             return self.TYPE
+        elif identifier is None:
+            return None
         else:
             return self.rdfcalendar[identifier]
 
@@ -450,12 +472,12 @@ class Store:
         """
         Return a list of items filtered using information from 'filters', which
         consists of a list of 2-tuples, each containing a list of simple type
-        labels (eg. ["uid"], ["dtstart", "datetime"]) and either a corresponding
-        simple value (eg. "20050827") or a 2-tuple describing the comparison and
-        the value (eg. ("startswith", "200508")).
+        labels (eg. ["uid"], ["dtstart", "datetime"]) and either a list of
+        simple values (eg. ["20050827", "20050828"]) or a 2-tuple describing the
+        comparison and a single value (eg. ("startswith", "200508")).
         """
 
-        results = []
+        new_filters = []
         for names, values in filters:
 
             # Get the actual property types.
@@ -466,54 +488,57 @@ class Store:
 
             # Get suitably prepared values.
 
-            for value in values:
-                if isinstance(value, tuple):
-                    comparison, value = value
-                else:
-                    comparison = None
+            property_values = []
+            if isinstance(values, tuple):
+                comparison, value = values
                 property_value = self.get_value_from_identifier(property_types[-1], value)
-                results += self.get_owner_items_with_properties(property_types, property_value, comparison=comparison)
+                property_values.append(property_value)
+            else:
+                comparison = None
+                for value in values:
+                    property_value = self.get_value_from_identifier(property_types[-1], value)
+                    property_values.append(property_value)
+            new_filters.append((property_types, property_values, comparison))
 
-        return unique(results)
-
-    def get_filtered_values(self, filters):
-
-        """
-        Return a list of values filtered using information from 'filters', which
-        consists of a list of 2-tuples, each containing a simple type label (eg.
-        "uid", "datetime") and either a corresponding simple value (eg.
-        "20050827") or a 2-tuple describing the comparison and the value (eg.
-        ("startswith", "200508")).
-        """
+        if getattr(self.cstore.store, "supports_querying", 0):
+            return self.get_filtered_items_using_query(new_filters)
+        else:
+            return self.get_filtered_items_manually(new_filters)
 
+    def get_filtered_items_manually(self, filters):
         results = []
-        for names, values in filters:
-
-            # Get the actual property types.
-
-            property_types = []
-            for name in names:
-                property_types.append(self.get_property_type_from_identifier(name))
-
-            # Get suitably prepared values.
-
-            for value in values:
-                if isinstance(value, tuple):
-                    comparison, value = value
-                else:
-                    comparison = None
-                property_value = self.get_value_from_identifier(property_types[-1], value)
-                results += self.get_values_for_properties(property_types, property_value, comparison=comparison)
-
+        for property_types, property_values, comparison in filters:
+            results += self.get_owner_items_with_properties(property_types, property_values, comparison=comparison)
         return unique(results)
 
-    def combine_items(self, existing_items, items):
+    def get_filtered_items_using_query(self, filters):
+        queries = []
+        for property_types, property_values, comparison in filters:
+            queries.append(self.get_owner_items_using_query(property_types, property_values, comparison=comparison))
+        return self.cstore.store.union(queries)
+
+    def combine_items(self, results):
+        if getattr(self.cstore.store, "supports_querying", 0):
+            return self.combine_items_using_query(results)
+        else:
+            return self.combine_items_manually(results)
+
+    def combine_items_manually(self, results):
+        found_items = results[0]
+        for result in results[1:]:
+            found_items = self._combine_items_manually(found_items, result)
+        return found_items
+
+    def _combine_items_manually(self, existing_items, items):
         results = []
         for item in existing_items:
             if item in items:
                 results.append(item)
         return results
 
+    def combine_items_using_query(self, results):
+        return self.cstore.store.intersection(results)
+
     def get_items_using_attributes(self, attributes, default_item_types):
 
         """
@@ -522,6 +547,11 @@ class Store:
         return a list of items.
         """
 
+        # NOTE: Despite potentially flexible support for querying in sqltriples,
+        # NOTE: sqlite doesn't seem to support more than simple left-to-right
+        # NOTE: evaluation of union/intersect with select statements. Thus, we
+        # NOTE: must be careful with the criteria below.
+
         results = []
 
         # Use filtering categories.
@@ -540,9 +570,20 @@ class Store:
             elif attributes.get("filter-type") == "person":
                 filters.append((["organizer", "uri"], [filter_item_value]))
                 filters.append((["attendee", "uri"], [filter_item_value]))
-                filters.append((["sender", "uri"], [filter_item_value]))
-                filters.append((["email", "details"], [filter_item_value]))
 
+            # Or specify non-related-to items.
+            # NOTE: Hack used to optimise sqltriples-based querying.
+
+            elif getattr(self.cstore.store, "supports_querying", 0):
+                filters.append((["related-to"], []))
+
+            results.append(self.get_filtered_items(filters))
+
+        # NOTE: Hack used to optimise sqltriples-based querying.
+
+        elif getattr(self.cstore.store, "supports_querying", 0):
+            filters = []
+            filters.append((["related-to"], []))
             results.append(self.get_filtered_items(filters))
 
         # Date-based filtering.
@@ -557,18 +598,9 @@ class Store:
                 pattern = ("startswith", "%04d" % attributes["year"])
 
             filters = []
-            filters.append((["dtstart", "datetime"], [pattern]))
-            filters.append((["dtend", "datetime"], [pattern]))
-            filters.append((["created", "datetime"], [pattern]))
-            results.append(self.get_filtered_items(filters))
-
-        # Context-based filtering.
-        # NOTE: This actually uses the calendar name!
-
-        if attributes.has_key("context"):
-            context = attributes["context"]
-            filters = []
-            filters.append((["name"], [context]))
+            #filters.append((["dtstart", "datetime"], pattern))
+            #filters.append((["dtend", "datetime"], pattern))
+            filters.append(([None, "datetime"], pattern))
             results.append(self.get_filtered_items(filters))
 
         # Or use type-based criteria.
@@ -580,14 +612,12 @@ class Store:
 
         filters = []
         filters.append((["item-type"], item_types))
+
         results.append(self.get_filtered_items(filters))
 
         # Combine the results.
 
-        found_items = results[0]
-        for result in results[1:]:
-            found_items = self.combine_items(found_items, result)
-        return found_items
+        return self.combine_items(results)
 
 def open(store_name, store_type, base_uri=None, context=None, **kw):
     store = RDFCalendar.Store.open(store_name, store_type, context, **kw)
