@@ -271,15 +271,18 @@ class ItemSelectorResource:
 
         item_value = pvpath.split("/")[-1]
 
-        # Get the item identifier, fully decoding the component.
-        # Override filtering if more than one item value has been encountered.
+        # Either get the item identifier, fully decoding the component.
 
-        if attributes.has_key("filter-type") and not attributes.has_key("filter-item-value"):
-            attributes["filter-item-value"] = urllib.unquote(item_value)
-            del attributes["item-type"]
-        else:
+        if not attributes.has_key("item-value"):
             attributes["item-value"] = urllib.unquote(item_value)
 
+        # Or indicate that the resource is not valid.
+
+        else:
+            trans.set_response_code(404)
+            trans.get_response_stream().write("Resource not found")
+            raise EndOfResponse
+
         # Handle situations which do not include extra non-identifier
         # information.
 
@@ -298,7 +301,7 @@ class ItemSelectorResource:
 
                 # Or serve the view of the resource.
 
-                elif attributes.has_key("filter-type"):
+                elif attributes.has_key("filter-type") and not attributes.has_key("item-value"):
                     self.viewer.respond(trans)
                     raise EndOfResponse
 
@@ -318,7 +321,7 @@ class ItemSelectorResource:
 
                 # Detect filter view actions.
 
-                if attributes.has_key("filter-type"):
+                if attributes.has_key("filter-type") and not attributes.has_key("item-value"):
                     self.viewer.respond(trans)
                     raise EndOfResponse
                 else:
@@ -381,9 +384,10 @@ class FilterSelectorResource:
 
     path_encoding = "utf-8"
 
-    def __init__(self, filter_type, viewer, default_item_type):
+    def __init__(self, filter_type, viewer, prop_viewer, default_item_type=None):
         self.filter_type = filter_type
         self.viewer = viewer
+        self.prop_viewer = prop_viewer
         self.default_item_type = default_item_type
 
     def respond(self, trans):
@@ -393,13 +397,43 @@ class FilterSelectorResource:
         resource.
         """
 
+        vpath = trans.get_virtual_path_info(self.path_encoding)
+        vpath_fields = vpath.split("/")
+
         # Start recording useful information.
 
         attributes = trans.get_attributes()
         attributes["filter-type"] = self.filter_type
-        attributes["item-type"] = self.default_item_type
-        self.viewer.respond(trans)
-        raise EndOfResponse
+
+        if len(vpath_fields) != 1 and vpath_fields[1]:
+
+            # Expect a doubly-"URL encoded" identifier.
+
+            item_value = vpath_fields[1]
+            attributes["filter-item-value"] = urllib.unquote(item_value)
+            trans.set_virtual_path_info("/" + "/".join(vpath_fields[2:]))
+
+            # Let the viewer decide about the structure below this resource.
+
+            self.viewer.respond(trans)
+            raise EndOfResponse
+
+        else:
+
+            # Set any restricting item type criteria.
+
+            if self.default_item_type is not None:
+                attributes["item-type"] = self.default_item_type
+
+            # Either present the items in a property view or give an error.
+
+            if trans.get_request_method() == "PROPFIND":
+                self.prop_viewer.respond(trans)
+                raise EndOfResponse
+            else:
+                trans.set_response_code(405)
+                trans.get_response_stream().write("Method not allowed")
+                raise EndOfResponse
 
 class ExporterSelectorResource:
 
