@@ -22,11 +22,13 @@ If not, write to the Free Software Foundation, Inc.,
 """
 
 from WebStack.Generic import ContentType, EndOfResponse
-from WebCalendar.Exporter import export_items
 import XSLForms.Resources.WebResources
 import XSLForms.Utils
 from XSLTools import XMLCalendar
+from WebCalendar.Exporter import export_items
+from WebCalendar.Common import default_item_types
 import os
+import time # for default filter information
 
 # Standard Web-oriented resource classes.
 
@@ -49,8 +51,6 @@ class CalendarViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
 
     # View configuration.
 
-    default_item_types = ["event", "to-do", "card", "message"]
-
     def __init__(self, store):
         self.store = store
 
@@ -75,7 +75,7 @@ class CalendarViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
 
         # Find the items according to the attributes.
 
-        found_items = self.store.get_items_using_attributes(attributes, self.default_item_types)
+        found_items = self.store.get_items_using_attributes(attributes, default_item_types)
 
         # Check for download actions.
 
@@ -87,26 +87,22 @@ class CalendarViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
             raise EndOfResponse
 
         elif parameters.has_key("new-event"):
-            if attributes.has_key("filter-type"):
-                new_path = trans.update_path(pvpath, "../../event/new/")
-            else:
-                new_path = trans.update_path(pvpath, "../event/new/")
+            new_path = trans.update_path(pvpath, "../event/new/")
             trans.redirect(trans.encode_path(new_path, self.path_encoding))
             # Response ended!
 
         elif parameters.has_key("new-to-do"):
-            if attributes.has_key("filter-type"):
-                new_path = trans.update_path(pvpath, "../../to-do/new/")
-            else:
-                new_path = trans.update_path(pvpath, "../to-do/new/")
+            new_path = trans.update_path(pvpath, "../to-do/new/")
             trans.redirect(trans.encode_path(new_path, self.path_encoding))
             # Response ended!
 
         elif parameters.has_key("new-card"):
-            if attributes.has_key("filter-type"):
-                new_path = trans.update_path(pvpath, "../../card/new/")
-            else:
-                new_path = trans.update_path(pvpath, "../card/new/")
+            new_path = trans.update_path(pvpath, "../card/new/")
+            trans.redirect(trans.encode_path(new_path, self.path_encoding))
+            # Response ended!
+
+        elif parameters.has_key("new-journal"):
+            new_path = trans.update_path(pvpath, "../journal/new/")
             trans.redirect(trans.encode_path(new_path, self.path_encoding))
             # Response ended!
 
@@ -186,8 +182,6 @@ class ListViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
 
     # View configuration.
 
-    default_item_types = ["event", "to-do", "card", "message"]
-
     def __init__(self, store):
         self.store = store
 
@@ -240,7 +234,7 @@ class ListViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
 
         # Filter according to supplied criteria.
 
-        found_items = self.store.get_items_using_attributes(attributes, self.default_item_types)
+        found_items = self.store.get_items_using_attributes(attributes, default_item_types)
 
         # Check for download actions or editing actions.
 
@@ -259,26 +253,22 @@ class ListViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
             raise EndOfResponse
 
         elif parameters.has_key("new-event"):
-            if attributes.has_key("filter-type"):
-                new_path = trans.update_path(pvpath, "../../event/new/")
-            else:
-                new_path = trans.update_path(pvpath, "../event/new/")
+            new_path = trans.update_path(pvpath, "../event/new/")
             trans.redirect(trans.encode_path(new_path, self.path_encoding))
             # Response ended!
 
         elif parameters.has_key("new-to-do"):
-            if attributes.has_key("filter-type"):
-                new_path = trans.update_path(pvpath, "../../to-do/new/")
-            else:
-                new_path = trans.update_path(pvpath, "../to-do/new/")
+            new_path = trans.update_path(pvpath, "../to-do/new/")
             trans.redirect(trans.encode_path(new_path, self.path_encoding))
             # Response ended!
 
         elif parameters.has_key("new-card"):
-            if attributes.has_key("filter-type"):
-                new_path = trans.update_path(pvpath, "../../card/new/")
-            else:
-                new_path = trans.update_path(pvpath, "../card/new/")
+            new_path = trans.update_path(pvpath, "../card/new/")
+            trans.redirect(trans.encode_path(new_path, self.path_encoding))
+            # Response ended!
+
+        elif parameters.has_key("new-journal"):
+            new_path = trans.update_path(pvpath, "../journal/new/")
             trans.redirect(trans.encode_path(new_path, self.path_encoding))
             # Response ended!
 
@@ -353,8 +343,6 @@ class PropertyViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
 
     # View configuration.
 
-    default_item_types = ["event", "to-do", "card", "message"]
-
     def __init__(self, store):
         self.store = store
 
@@ -400,7 +388,7 @@ class PropertyViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
 
         else:
 
-            found_items = self.store.get_items_using_attributes(attributes, self.default_item_types)
+            found_items = self.store.get_items_using_attributes(attributes, default_item_types)
             set_filter_attributes(items_root, attributes)
 
             # Set special attributes in order to indicate different kinds of
@@ -472,11 +460,11 @@ class SimpleViewerResource(XSLForms.Resources.WebResources.XSLFormsResource):
 
         doc = form.new_document(self.default_document)
         top = doc.xpath("*")[0]
-        for directory in self.get_directories(pvpath):
+        for directory in self.get_directories(trans, pvpath):
             e = doc.createElement("directory")
             e.setAttribute("name", directory)
             top.appendChild(e)
-        for file in self.get_files(pvpath):
+        for file in self.get_files(trans, pvpath):
             e = doc.createElement("file")
             e.setAttribute("name", file)
             top.appendChild(e)
@@ -508,10 +496,10 @@ class MonthViewerResource(SimpleViewerResource):
         }
     default_document = "months"
 
-    def get_directories(self, pvpath):
+    def get_directories(self, trans, pvpath):
         return map(str, range(1, 13))
 
-    def get_files(self, pvpath):
+    def get_files(self, trans, pvpath):
         return []
 
 class RootViewerResource(SimpleViewerResource):
@@ -523,14 +511,17 @@ class RootViewerResource(SimpleViewerResource):
         }
     default_document = "root"
 
-    def get_directories(self, pvpath):
-        d = ["all", "event", "message", "person", "related-to", "to-do"]
+    def get_directories(self, trans, pvpath):
+        d = ["all", "event", "journal", "message", "to-do"]
         if pvpath == "/":
             d.append("calendar")
             d.append("card")
+        if not trans.get_attributes().has_key("filter-type"):
+            d.append("related-to")
+            d.append("person")
         return d
 
-    def get_files(self, pvpath):
+    def get_files(self, trans, pvpath):
         return ["all.ics", "event.ics", "to-do.ics"]
 
 # Common functions.
@@ -541,12 +532,20 @@ def set_filter_attributes(element, attributes):
 
     if attributes.has_key("filter-type"):
         element.setAttribute("filter-type", attributes["filter-type"])
-    elif attributes.has_key("item-type"):
-        element.setAttribute("filter-type", attributes["item-type"])
-    else:
-        element.setAttribute("filter-type", "all")
+    if attributes.has_key("item-type"):
+        element.setAttribute("item-type", attributes["item-type"])
+
+    if attributes.has_key("year"):
+        element.setAttribute("year", str(attributes["year"]))
+    if attributes.has_key("month"):
+        element.setAttribute("month", str(attributes["month"]))
+
+    year_now, month_now = time.gmtime()[0:2]
+    element.setAttribute("year-now", str(year_now))
+    element.setAttribute("month-now", str(month_now))
 
     if attributes.has_key("filter-item-value"):
+        element.setAttribute("filter-item-value", attributes["filter-item-value"])
         if attributes.get("filter-type") == "related-to":
             element.setAttribute("related-to", attributes["filter-item-value"])
             element.setAttribute("filtered", "yes")
