@@ -21,7 +21,7 @@ If not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 """
 
-__version__ = "0.1.3"
+__version__ = "0.1.4"
 
 import os
 
@@ -63,23 +63,22 @@ def get_site(store):
     # Here's what the site looks like:
 
     """
-    --> date_choices -------------> item_choices ---------------------> view_choices --> ItemSelectorResource
-        |                        |  | | | |                        | |  |
-        | /date                  |  | | | | /<type>                | |  | /
-        |                        |  | | | |                        | |  |
-        -> DateSelectorResource --  | | | -> TypeSelectorResource -- |  -> ViewSelectorResource
-                                    | | |                            |
-                                    | | | /<filter>                  |
-                                    | | |                            |
-                                    | | -> FilterSelectorResource ----
-                                    | |  
-                                    | | /<filename>
-                                    | |
-                                    | -> ExporterSelectorResource
-                                    |
-                                    | /
-                                    |
-                                    -> TopSelectorResource
+    --> date_choices -------------> filter_choices -------------> item_choices -----------------> view_choices --> ItemSelectorResource
+        |                        |  |                          |  | | |                        |  |
+        | /date                  |  | /<filter>                |  | | | /<type>                |  | /
+        |                        |  |                          |  | | |                        |  |
+        -> DateSelectorResource --  -> FilterSelectorResource --  | | -> TypeSelectorResource --  -> ViewSelectorResource
+                                                                  | |  
+                                                                  | | /<filename>
+                                                                  | |
+                                                                  | -> ExporterSelectorResource
+                                                                  |
+                                                                  | /
+                                                                  |
+                                                                  -> TopSelectorResource
+
+        Set year, month             Set filter-type,              Set item-type
+                                    filter-item-value
     """
 
     # Make a simple Web site.
@@ -89,7 +88,7 @@ def get_site(store):
         "scripts" : scripts_dir,
         "images" : images_dir,
         "" : ViewSelectorResource(list_viewer_resource, calendar_viewer_resource, prop_viewer_resource)
-        })
+        }) # not pass_through: identifiers are removed from the vpath
 
     # Handle access to the above where identifiers are present.
     # (This also happens when styles and scripts are being accessed through an
@@ -103,24 +102,28 @@ def get_site(store):
         "event" : TypeSelectorResource(view_choices),
         "to-do" : TypeSelectorResource(view_choices),
         "card" : TypeSelectorResource(view_choices),
+        "journal" : TypeSelectorResource(view_choices),
         "message" : TypeSelectorResource(view_choices),
         "calendar" : TypeSelectorResource(view_choices),
-        # URI selectors producing views...
-        "related-to" : FilterSelectorResource("related-to", view_choices, "to-do"),
-        "person" : FilterSelectorResource("person", view_choices, "card"),
         # Exporters...
         "event.ics" : ExporterSelectorResource("event", collection_exporter_resource, prop_viewer_resource),
         "to-do.ics" : ExporterSelectorResource("to-do", collection_exporter_resource, prop_viewer_resource),
         "all.ics" : ExporterSelectorResource("all", collection_exporter_resource, prop_viewer_resource),
         # Views...
         "all" : view_choices,
-        "" : TopSelectorResource("all/", webdav_root),
-        None : view_choices
+        "" : TopSelectorResource("all/", webdav_root)
+        }, pass_through=1)
+
+    filter_choices = MapResource({
+        # URI selectors producing views...
+        "related-to" : FilterSelectorResource("related-to", item_choices, prop_viewer_resource),
+        "person" : FilterSelectorResource("person", item_choices, prop_viewer_resource, "card"),
+        None : item_choices
         }, pass_through=1)
 
     date_choices = MapResource({
-        "date" : DateSelectorResource(item_choices, webdav_months),
-        None : item_choices
+        "date" : DateSelectorResource(filter_choices, webdav_months),
+        None : filter_choices
         }, pass_through=1)
 
     return date_choices
